

/*
*******************************************************************************
Include Part
*******************************************************************************
*/
#include "lcd.h"
#include "util.h"
#include "main.h"
/*
*******************************************************************************
Define Part
*******************************************************************************
*/
#define LCD_EN_0 cbi(PORTE,PE4)
#define LCD_EN_1 sbi(PORTE,PE4)
#define LCD_RW_0 cbi(PORTE,PE3)
#define LCD_RS_0 cbi(PORTE,PE2)
#define LCD_RS_1 sbi(PORTE,PE2)
//
#define LCD_STEP_0_WAIT_TIME			0
#define LCD_STEP_1_WRITE_COMMAND		1
#define LCD_STEP_2_WRITE_STRING		2
#define LCD_STEP_3_WRITE_DATA			3
#define LCD_STEP_4_CHECK_DATA			4
#define LCD_STEP_5_WRITE_COMMAND		5
#define LCD_STEP_6_WRITE_STRING		6
#define LCD_STEP_7_WRITE_DATA			7
#define LCD_STEP_8_CHECK_DATA			8
#define LCD_STEP_9_WRITE_END			9
//
/*
********************************************************************************
Variable Declaration Part
********************************************************************************
*/
unsigned char lcd_1th_line[16];
unsigned char lcd_2th_line[16];
//
unsigned char old_lcd_step = LCD_STEP_0_WAIT_TIME;
unsigned char lcd_step = LCD_STEP_1_WRITE_COMMAND;
//
unsigned char lcd_data_count = 0;
unsigned char check_count = 0;
/*
********************************************************************************

********************************************************************************
*/
void LCD_INITIALIZE()
{
	WAIT_1MS(40);
	//
	LCD_WRITE_COMMAND(0x38);
	//
	WAIT_1MS(20);
	//
	LCD_WRITE_COMMAND(0x38);
	//
	WAIT_1MS(1);
	//
	LCD_WRITE_COMMAND(0x38);
	//
	LCD_WRITE_COMMAND(0x38);
	//
	LCD_WRITE_COMMAND(0x0f);
	//
	LCD_WRITE_COMMAND(0x06);
	//
	LCD_WRITE_COMMAND(0x02);
	//
	LCD_WRITE_COMMAND(0x01);
}
/*
********************************************************************************

********************************************************************************
*/
void LCD_CLEAR_SCREEN()
{
	LCD_WRITE_COMMAND(0x01);
	WAIT_1MS(5);
}
/*
********************************************************************************

********************************************************************************
*/
void LCD_CURSOR_OFF()
{
	LCD_WRITE_COMMAND(0x0c);
	WAIT_1MS(5);
}
/*
********************************************************************************

********************************************************************************
*/
void LCD_WRITE_COMMAND(unsigned char lcd_command)
{
	LCD_EN_0;
	LCD_RW_0;
	LCD_RS_0;
	//
	LCD_EN_1;
	LCD_RW_0;
	LCD_RS_0;
	//
	PORTF = lcd_command;
	//
	LCD_EN_0;
	LCD_RW_0;
	LCD_RS_0;
}
/*
********************************************************************************

********************************************************************************
*/
void LCD_WRITE_DATA(unsigned char lcd_data)
{
	LCD_EN_0;
	LCD_RW_0;
	LCD_RS_1;
	//
	LCD_EN_1;
	LCD_RW_0;
	LCD_RS_1;
	//
	PORTF = lcd_data;
	//
	LCD_EN_0;
	LCD_RW_0;
	LCD_RS_1;
}
/*
********************************************************************************

********************************************************************************
*/
void LCD_DISPLAY_STRING(unsigned char *str)
{
	while(*str)
	{
		LCD_WRITE_DATA(*str++);
		WAIT_1MS(5);
	}
}
/*
********************************************************************************

********************************************************************************
*/
void CHECK_TIMER_LCD()
{
	if(timer_lcd == 0)
	{
		lcd_step = old_lcd_step;
		lcd_step++;
	}
}
/*
********************************************************************************

********************************************************************************
*/
void START_TIMER_LCD()
{
	old_lcd_step = lcd_step;
	lcd_step = LCD_STEP_0_WAIT_TIME;
	//
	//Է°  3 ð ҿ : 2*3 = 6ms
	timer_lcd = 2;
}
/*
********************************************************************************

********************************************************************************
*/
void INIT_LCD()
{
	LCD_INITIALIZE();
	LCD_CLEAR_SCREEN();
	LCD_WRITE_COMMAND(0x80);
	WAIT_1MS(5);
	LCD_DISPLAY_STRING((unsigned char *)"Firmtech.co.,LTD");
	LCD_WRITE_COMMAND(0xc0);
	WAIT_1MS(5);
	LCD_DISPLAY_STRING((unsigned char *)"FZ800ED_PARSING ");
	LCD_CURSOR_OFF();
}
/*
********************************************************************************
LCD_PROCESS
********************************************************************************
*/
void LCD_PROCESS()
{
	//ADC (  ) ̿Ͽ LCD Ѵ.
	if((operation_adc_number> 0)&&(operation_adc_process == OPERATION_OK))
	{
		switch(lcd_step)
		{
			case LCD_STEP_0_WAIT_TIME:
				CHECK_TIMER_LCD();
				break;
			case LCD_STEP_1_WRITE_COMMAND:
				LCD_WRITE_COMMAND(0x80);
				START_TIMER_LCD();
				break;
			case LCD_STEP_2_WRITE_STRING:
				memset(lcd_1th_line,0x20,16);
				memcpy(lcd_1th_line,"Cds Value:",10);
				//
				lcd_1th_line[10] = '0';
				lcd_1th_line[11] = 'x';
				lcd_1th_line[12] = operation_adc_value[0];
				lcd_1th_line[13] = operation_adc_value[1];
				lcd_1th_line[14] = operation_adc_value[2];
				lcd_1th_line[15] = operation_adc_value[3];
				//
				lcd_step++;
				break;
			case LCD_STEP_3_WRITE_DATA:
				LCD_WRITE_DATA(lcd_1th_line[lcd_data_count]);
				START_TIMER_LCD();
				break;
			case LCD_STEP_4_CHECK_DATA:
				if(lcd_data_count <15)
				{
					lcd_data_count++;
					lcd_step = LCD_STEP_3_WRITE_DATA;
				}
				else
				{
					lcd_data_count = 0;
					lcd_step = LCD_STEP_5_WRITE_COMMAND;
				}
				break;
			case LCD_STEP_5_WRITE_COMMAND:
				LCD_WRITE_COMMAND(0xc0);
				START_TIMER_LCD();
				break;
			case LCD_STEP_6_WRITE_STRING:
				memset(lcd_2th_line,0x20,16);
				memcpy(lcd_2th_line,"================",16);
				//
				//ADC ͸  , '>' ǥð right shift 1ȸ 
				check_count++;
				if(check_count > 15)
					check_count = 0;
				lcd_2th_line[check_count] = '>';
				//
				lcd_step++;
				break;
			case LCD_STEP_7_WRITE_DATA:
				LCD_WRITE_DATA(lcd_2th_line[lcd_data_count]);
				START_TIMER_LCD();
				break;
			case LCD_STEP_8_CHECK_DATA:
				if(lcd_data_count <15)
				{
					lcd_data_count++;
					lcd_step = LCD_STEP_7_WRITE_DATA;
				}
				else
				{
					lcd_data_count = 0;
					lcd_step = LCD_STEP_9_WRITE_END;
				}
				break;
			case LCD_STEP_9_WRITE_END:
				old_lcd_step = LCD_STEP_0_WAIT_TIME;
				lcd_step = LCD_STEP_1_WRITE_COMMAND;
				//
				operation_adc_number = 0;
				operation_adc_process = OPERATION_NO;
				break;
			default:
				break;
		}
	}
}


