/*
*******************************************************************************
Include Part
*******************************************************************************
*/
#include "main.h"
#include "util.h"
/*
*******************************************************************************
Define Part
*******************************************************************************
*/

/*
********************************************************************************
Function For UART0
********************************************************************************
*/
SIGNAL(SIG_UART0_RECV)
{
	rx0_buf[p_rx0_wr++] = UDR0;
	if(rx0_buf[p_rx0_wr-1] == CR_VALUE)cr0_check_flag++;
	if(rx0_buf[p_rx0_wr-1] == LF_VALUE)lf_check_flag++;
	if (p_rx0_wr > UART0_BUF_SIZE-1)p_rx0_wr = 0;
}
//
void INIT_UART0(unsigned char baudrate)
{
	unsigned int i;
	
	UBRR0H      = 0;
	if(baudrate == BAUD_9600){UBRR0L = 71;}/* 9600 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_19200){UBRR0L = 35;}/* 19200 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_38400){UBRR0L = 17;}/* 38400 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_57600){UBRR0L = 11;}/* 57600 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_115200){UBRR0L = 5;} /* 115200 BAUD at 11.0592MHz*/
	else {UBRR0L = 71;};/* default 9600 BAUD at 11.0592MHz*/
	UCSR0B = (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);
	p_rx0_wr = 0;
	p_rx0_rd = 0;
	for (i=0; i<UART0_BUF_SIZE; i++) rx0_buf[i] = 0;
 }
//
unsigned char PUTCHAR_UART0 (unsigned char c)
{
	while (!(UCSR0A & 0x20)) ;
	UDR0 = c;
	return 0;
}
//
void DISPSTR_UART0 (unsigned char *s)
{
	while (*s != '\0') 
		PUTCHAR_UART0(*s++);
}
//
unsigned int CHECK_RX_BUF_UART0(void)
{
	unsigned int len;

	if (p_rx0_wr == p_rx0_rd)
	{
		len = 0;
	}
	else
	{
		if (p_rx0_wr > p_rx0_rd) len = p_rx0_wr - p_rx0_rd;
		else len = UART0_BUF_SIZE + p_rx0_wr - p_rx0_rd;
	}
	return len;
}
//
unsigned char GETCHAR_UART0 (void)
{
	unsigned char ch;
    
	ch = rx0_buf[p_rx0_rd];
	p_rx0_rd++;
	if (p_rx0_rd > UART0_BUF_SIZE-1) p_rx0_rd = 0;
	return ch;
}
//
void DISPLAY_CR_LF_UART0()
{
	PUTCHAR_UART0(0x0d);
	PUTCHAR_UART0(0x0a);
}
/*
********************************************************************************
Function For UART1
********************************************************************************
*/
SIGNAL(SIG_UART1_RECV)
{
	rx1_buf[p_rx1_wr++] = UDR1;
	if(rx1_buf[p_rx1_wr-1] == CR_VALUE)cr1_check_flag++;
	if (p_rx1_wr > UART1_BUF_SIZE-1)p_rx1_wr = 0;
}
//
void INIT_UART1(unsigned char baudrate)
{
	unsigned int i;

	UBRR1H      = 0;
	if(baudrate == BAUD_9600){UBRR1L = 71;}/* 9600 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_19200){UBRR1L = 35;}/* 19200 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_38400){UBRR1L = 17;}/* 38400 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_57600){UBRR1L = 11;}/* 57600 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_115200){UBRR1L = 5;} /* 115200 BAUD at 11.0592MHz*/
	else {UBRR1L = 71;};/* default 9600 BAUD at 11.0592MHz*/
	UCSR1B = (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);
	p_rx1_wr = 0;
	p_rx1_rd = 0;
	for (i=0; i<UART1_BUF_SIZE; i++) rx1_buf[i] = 0;
}
//
unsigned char PUTCHAR_UART1 (unsigned char c)
{
	while (!(UCSR1A & 0x20)) ;
	UDR1 = c;
	return 0;
}
//
void DISPSTR_UART1 (unsigned char *s)
{
	while (*s != '\0') 
		PUTCHAR_UART1(*s++);
}
//
unsigned int CHECK_RX_BUF_UART1(void)
{
	unsigned int len;

	if (p_rx1_wr == p_rx1_rd)
	{
		len = 0;
	}
	else
	{
		if (p_rx1_wr > p_rx1_rd) len = p_rx1_wr - p_rx1_rd;
		else len = UART1_BUF_SIZE + p_rx1_wr - p_rx1_rd;
	}
	return len;
}
//
unsigned char GETCHAR_UART1 (void)
{
	unsigned char ch;
    
	ch = rx1_buf[p_rx1_rd];
	p_rx1_rd++;
	if (p_rx1_rd > UART1_BUF_SIZE-1) p_rx1_rd = 0;
	return ch;
}
//
void DISPLAY_CR_LF_UART1()
{
	PUTCHAR_UART1(0x0d);
	PUTCHAR_UART1(0x0a);
}
/*
********************************************************************************
Function For Timer0
********************************************************************************
*/
SIGNAL(SIG_OVERFLOW0)
{
	timer0_counter++;
	if (timer0_counter > 200 )
	{	
		timer0_counter = 0;
	}
}
//
void INIT_TIMER0(void)
{
	TIMSK |= 1 << TOIE0;
	TCNT0 = 0;
	TCCR0 = 5;
	timer0_counter = 0;
}
/*
********************************************************************************
CONVERT HEX INTO CHAR
********************************************************************************
*/
unsigned char HEX2CHAR(unsigned char c)
{
	if ((c == 0||c > 0) && (c < 10))
		return '0' + c;
	if (c >= 10 && c <= 15)
		return 'A' + c - 10;
	//
	return c;
}
/*
********************************************************************************
Function For Delay
********************************************************************************
*/
void WAIT_1MS(unsigned int cnt)
{
	unsigned int i;

	for (i = 0; i < cnt; i++) WAIT_1US(1000);
}
//
void WAIT_1US(unsigned int cnt)
{
	unsigned int i;

	for (i = 0; i < cnt; i++) ;
}





